// SleepTune - pauses iTunes after a user-defined period of time.
// Copyright (C) 2005  Martey Dodoo
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using System.ComponentModel;
using System.Diagnostics;
using iTunesLib;

namespace SleepTune
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.NumericUpDown numSleepTime;
		private System.Windows.Forms.Button btnAbout;
		private System.Windows.Forms.Label lblMinutes1;
		private System.Windows.Forms.RadioButton radioMakeiTunesSleep;
		private System.Windows.Forms.Label labelMinutes2;
		private System.Windows.Forms.MenuItem mnuModify;
		private System.Timers.Timer timer;
		private System.Windows.Forms.MenuItem mnuCancel;
		private System.Windows.Forms.ContextMenu contextMenu;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.NumericUpDown numWakeTime;
		private System.Windows.Forms.RadioButton radioMakeiTunesPlay;
		
		private NotifyIcon trayIcon;
		private iTunesLib.IiTunes iTunesApp;
		private int intTime;
		private string strAction;
		
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			trayIcon = new NotifyIcon();
			trayIcon.Text = Application.ProductName;
			trayIcon.Visible = true;
			System.IO.Stream icoStream = this.GetType().Assembly.GetManifestResourceStream("sound.ico");
			trayIcon.Icon = new Icon(icoStream);
			trayIcon.ContextMenu = contextMenu;
		}
		
		[STAThread]
		public static void Main(string[] args)
		{
			#region mutex creation
			/// http://www.codeproject.com/csharp/SingleInstanceApplication.asp
			bool mutexCreated = false;
			// The name of the mutex is to be prefixed with Local\ to make
			// sure that its is created in the per-session namespace,
			// not in the global namespace.
			string mutexName = "Local\\" + 
			  System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
			Mutex mutex = null;
			try{
				mutex = new Mutex(false, mutexName, out mutexCreated);
			}catch(Exception ex){
				MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace + "\n\n" + 
				  "Could not create mutex! Application must now exit.",
				  "Mutex Creation Error",MessageBoxButtons.OK,
				  MessageBoxIcon.Error);
				Application.Exit();
			}
			#endregion
			if(mutexCreated && mutex.WaitOne()){
				Application.Run(new MainForm());
				#region mutex destruction
				GC.KeepAlive(mutex);
				try{
					mutex.ReleaseMutex();
				}catch(ApplicationException ex){
					MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace+"\n\n" +
					  "Could not release mutex! Application will continue to " +
					  "exit.","Mutex Release Error",MessageBoxButtons.OK,
					  MessageBoxIcon.Error);
					GC.Collect();
				}
				#endregion
			}else{
				MessageBox.Show("SleepTune is already running.","SleepTune",
				  MessageBoxButtons.OK,MessageBoxIcon.Information);
			}
			mutex.Close();
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.radioMakeiTunesPlay = new System.Windows.Forms.RadioButton();
			this.numWakeTime = new System.Windows.Forms.NumericUpDown();
			this.btnCancel = new System.Windows.Forms.Button();
			this.contextMenu = new System.Windows.Forms.ContextMenu();
			this.mnuCancel = new System.Windows.Forms.MenuItem();
			this.timer = new System.Timers.Timer();
			this.mnuModify = new System.Windows.Forms.MenuItem();
			this.labelMinutes2 = new System.Windows.Forms.Label();
			this.radioMakeiTunesSleep = new System.Windows.Forms.RadioButton();
			this.lblMinutes1 = new System.Windows.Forms.Label();
			this.btnAbout = new System.Windows.Forms.Button();
			this.numSleepTime = new System.Windows.Forms.NumericUpDown();
			this.btnOK = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.numWakeTime)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.timer)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numSleepTime)).BeginInit();
			this.SuspendLayout();
			// 
			// radioMakeiTunesPlay
			// 
			this.radioMakeiTunesPlay.Location = new System.Drawing.Point(16, 56);
			this.radioMakeiTunesPlay.Name = "radioMakeiTunesPlay";
			this.radioMakeiTunesPlay.Size = new System.Drawing.Size(136, 24);
			this.radioMakeiTunesPlay.TabIndex = 4;
			this.radioMakeiTunesPlay.Text = "Make iTunes play in";
			// 
			// numWakeTime
			// 
			this.numWakeTime.Location = new System.Drawing.Point(160, 56);
			this.numWakeTime.Maximum = new System.Decimal(new int[] {
						360,
						0,
						0,
						0});
			this.numWakeTime.Name = "numWakeTime";
			this.numWakeTime.Size = new System.Drawing.Size(48, 20);
			this.numWakeTime.TabIndex = 5;
			this.numWakeTime.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.numWakeTime.Value = new System.Decimal(new int[] {
						30,
						0,
						0,
						0});
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Image = ((System.Drawing.Image)(resources.GetObject("btnCancel.Image")));
			this.btnCancel.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnCancel.Location = new System.Drawing.Point(112, 88);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(88, 32);
			this.btnCancel.TabIndex = 8;
			this.btnCancel.Text = "&Cancel";
			this.btnCancel.Click += new System.EventHandler(this.BtnCancelClick);
			// 
			// contextMenu
			// 
			this.contextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.mnuModify,
						this.mnuCancel});
			// 
			// mnuCancel
			// 
			this.mnuCancel.Index = 1;
			this.mnuCancel.Text = "Cancel";
			this.mnuCancel.Click += new System.EventHandler(this.BtnCancelClick);
			// 
			// timer
			// 
			this.timer.Interval = 60000;
			this.timer.SynchronizingObject = this;
			this.timer.Elapsed += new System.Timers.ElapsedEventHandler(this.TimerElapsed);
			// 
			// mnuModify
			// 
			this.mnuModify.Index = 0;
			this.mnuModify.Text = "Modify";
			this.mnuModify.Click += new System.EventHandler(this.mnuModifyClick);
			// 
			// labelMinutes2
			// 
			this.labelMinutes2.Location = new System.Drawing.Point(224, 60);
			this.labelMinutes2.Name = "labelMinutes2";
			this.labelMinutes2.Size = new System.Drawing.Size(56, 16);
			this.labelMinutes2.TabIndex = 6;
			this.labelMinutes2.Text = "minutes";
			// 
			// radioMakeiTunesSleep
			// 
			this.radioMakeiTunesSleep.Checked = true;
			this.radioMakeiTunesSleep.Location = new System.Drawing.Point(16, 16);
			this.radioMakeiTunesSleep.Name = "radioMakeiTunesSleep";
			this.radioMakeiTunesSleep.Size = new System.Drawing.Size(136, 24);
			this.radioMakeiTunesSleep.TabIndex = 1;
			this.radioMakeiTunesSleep.TabStop = true;
			this.radioMakeiTunesSleep.Text = "Make iTunes sleep in";
			// 
			// lblMinutes1
			// 
			this.lblMinutes1.Location = new System.Drawing.Point(224, 20);
			this.lblMinutes1.Name = "lblMinutes1";
			this.lblMinutes1.Size = new System.Drawing.Size(56, 16);
			this.lblMinutes1.TabIndex = 3;
			this.lblMinutes1.Text = "minutes";
			// 
			// btnAbout
			// 
			this.btnAbout.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnAbout.Image = ((System.Drawing.Image)(resources.GetObject("btnAbout.Image")));
			this.btnAbout.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnAbout.Location = new System.Drawing.Point(216, 88);
			this.btnAbout.Name = "btnAbout";
			this.btnAbout.Size = new System.Drawing.Size(88, 32);
			this.btnAbout.TabIndex = 9;
			this.btnAbout.Text = "&About";
			this.btnAbout.Click += new System.EventHandler(this.BtnAboutClick);
			// 
			// numSleepTime
			// 
			this.numSleepTime.Location = new System.Drawing.Point(160, 16);
			this.numSleepTime.Maximum = new System.Decimal(new int[] {
						360,
						0,
						0,
						0});
			this.numSleepTime.Name = "numSleepTime";
			this.numSleepTime.Size = new System.Drawing.Size(48, 20);
			this.numSleepTime.TabIndex = 2;
			this.numSleepTime.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.numSleepTime.Value = new System.Decimal(new int[] {
						30,
						0,
						0,
						0});
			// 
			// btnOK
			// 
			this.btnOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Image = ((System.Drawing.Image)(resources.GetObject("btnOK.Image")));
			this.btnOK.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnOK.Location = new System.Drawing.Point(8, 88);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(88, 32);
			this.btnOK.TabIndex = 7;
			this.btnOK.Text = "&OK";
			this.btnOK.Click += new System.EventHandler(this.BtnOKClick);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(312, 125);
			this.Controls.Add(this.numWakeTime);
			this.Controls.Add(this.labelMinutes2);
			this.Controls.Add(this.radioMakeiTunesPlay);
			this.Controls.Add(this.radioMakeiTunesSleep);
			this.Controls.Add(this.lblMinutes1);
			this.Controls.Add(this.numSleepTime);
			this.Controls.Add(this.btnAbout);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MainForm";
			this.ShowInTaskbar = false;
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.Text = "SleepTune";
			this.TopMost = true;
			((System.ComponentModel.ISupportInitialize)(this.numWakeTime)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.timer)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numSleepTime)).EndInit();
			this.ResumeLayout(false);
		}
		#endregion
		
		/// <summary>
		/// This is supposed to remove trayIcon when the application closes.
		/// </summary>
		protected override void Dispose(bool disposing){
			if(disposing){
				this.trayIcon.Dispose();
			}
			base.Dispose(disposing);
		}
		
		void BtnOKClick(object sender, System.EventArgs e)
		{
			if(radioMakeiTunesSleep.Checked){
				intTime = (int)numSleepTime.Value;
				strAction = "sleep";
			}else if(radioMakeiTunesPlay.Checked){
				intTime = (int)numWakeTime.Value;
				strAction = "wake";
			}
			// disable radio buttons to prevent tampering
			radioMakeiTunesSleep.Enabled = false;
			radioMakeiTunesPlay.Enabled = false;
			timer.Enabled = true;
			trayIcon.Text = Application.ProductName + ": " + strAction + 
			  " in " + intTime.ToString() + " minutes";
			this.Hide();
		}
		
		void BtnCancelClick(object sender, System.EventArgs e)
		{
			Close();
		}
		
		void BtnAboutClick(object sender, System.EventArgs e)
		{
			Process pBrowser = new Process();
			pBrowser.StartInfo.FileName = "http://www.marteydodoo.com/";
			pBrowser.Start();
		}
		
		void TimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			intTime = intTime - 1;
			switch (intTime){
			case 0:
				timer.Enabled = false;
				try{
					iTunesApp = new iTunesAppClass();;
				}catch(Exception ex){
					MessageBox.Show(ex.Message,"Could not find iTunes!",
					  MessageBoxButtons.OK,MessageBoxIcon.Error);
					Close();
				}
				// do action
				try{
					if(radioMakeiTunesSleep.Checked &&
					  (this.iTunesApp.PlayerState == iTunesLib.ITPlayerState.ITPlayerStatePlaying))
						iTunesApp.Pause();
					if(radioMakeiTunesPlay.Checked &&
					  (iTunesApp.PlayerState == iTunesLib.ITPlayerState.ITPlayerStateStopped))
						iTunesApp.Play();
				}catch(Exception ex){
					MessageBox.Show(ex.Message,"Error changing status of iTunes",
					  MessageBoxButtons.OK,MessageBoxIcon.Error);
				}
				Close();
				break;
			case 1:
				trayIcon.Text = Application.ProductName + ": " + strAction + 
				  " in " + intTime.ToString() + " minute";
				break;
			case 2:
				trayIcon.Text = Application.ProductName + ": " + strAction + 
				  " in " + intTime.ToString() + " minutes";
				break;
			default:
				trayIcon.Text = Application.ProductName + ": " + strAction + 
				  " in " + intTime.ToString() + " minutes";
				break;
			}
			if(radioMakeiTunesSleep.Checked){
				numSleepTime.Value = intTime;
			}else if(radioMakeiTunesPlay.Checked){
				numWakeTime.Value = intTime;
			}
		}
		
		void mnuModifyClick(object sender, System.EventArgs e)
		{
			this.Show();
		}
		
	}
}
